@@ -18,12 +18,12 @@ import copy
 import datetime
 import functools
 import multiprocessing
+import netaddr
 import operator
 import threading
 import types
 import uuid
 
-import netaddr
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net
 from neutron_lib.api.definitions import segment as segment_def
@@ -35,6 +35,7 @@ from neutron_lib import context as n_context
 from neutron_lib.db import api as db_api
 from neutron_lib import exceptions as n_exc
 from neutron_lib.exceptions import availability_zone as az_exc
+from neutron_lib.placement import constants as place_const
 from neutron_lib.placement import utils as place_utils
 from neutron_lib.plugins import directory
 from neutron_lib.plugins.ml2 import api
@@ -70,6 +71,7 @@ from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import impl_idl_ovn
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import maintenance
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_client
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_db_sync
+from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovs_fixes
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import worker
 from neutron import service
 from neutron.services.logapi.drivers.ovn import driver as log_driver
@@ -81,9 +83,6 @@ from neutron.services.trunk.drivers.ovn import trunk_driver
 LOG = log.getLogger(__name__)
 OVN_MIN_GENEVE_MAX_HEADER_SIZE = 38
 
-# TODO(ralonsoh): rehome this to ``neutron_lib.placement.constants``.
-ALLOCATION = 'allocation'
-
 
 class OVNPortUpdateError(n_exc.BadRequest):
     pass
@@ -306,10 +305,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         registry.subscribe(self.delete_segment_provnet_port,
                            resources.SEGMENT,
                            events.AFTER_DELETE)
-        # TODO(slaweq): use constant from neutron_lib.callbacks.resources once
-        # it will be available and released
         registry.subscribe(self._validate_allowed_address_pairs,
-                           'allowed_address_pair',
+                           resources.ALLOWED_ADDRESS_PAIR,
                            events.BEFORE_CREATE)
 
         # Handle security group/rule or address group notifications
@@ -419,6 +416,9 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._post_fork_event.clear()
         self._ovn_client_inst = None
 
+        # Patch python-ovs for fixes not yet released
+        ovs_fixes.apply_ovs_fixes()
+
         if worker_class == wsgi.WorkerService:
             self._setup_hash_ring()
 
@@ -451,7 +451,8 @@ class OVNMechanismDriver(api.MechanismDriver):
                 self.nb_ovn,
                 self.sb_ovn,
                 ovn_conf.get_ovn_neutron_sync_mode(),
-                self
+                self,
+                is_maintenance=True,
             )
             self.nb_synchronizer.sync()
 
@@ -1526,7 +1527,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         if uuid_ns is None:
             return False
         try:
-            allocation = context.current['binding:profile'][ALLOCATION]
+            allocation = context.current['binding:profile'][
+                place_const.ALLOCATION]
         except KeyError:
             return False
 
@@ -1609,14 +1611,11 @@ def delete_agent(self, context, id, _driver=None):
     chassis_name = agent['configurations']['chassis_name']
     _driver.sb_ovn.chassis_del(chassis_name, if_exists=True).execute(
         check_error=True)
-    if _driver.sb_ovn.is_table_present('Chassis_Private'):
-        # TODO(ralonsoh): implement the corresponding chassis_private
-        # commands in ovsdbapp.
-        try:
-            _driver.sb_ovn.db_destroy('Chassis_Private', chassis_name).execute(
-                check_error=True)
-        except idlutils.RowNotFound:
-            pass
+    try:
+        _driver.sb_ovn.db_destroy('Chassis_Private', chassis_name).execute(
+            check_error=True)
+    except idlutils.RowNotFound:
+        pass
     # Send a specific event that all API workers can get to delete the agent
     # from their caches. Ideally we could send a single transaction that both
     # created and deleted the key, but alas python-ovs is too "smart"
