@@ -13,8 +13,9 @@
 import abc
 from datetime import datetime
 import itertools
+import threading
+import time
 
-from eventlet import greenthread
 from neutron_lib.api.definitions import segment as segment_def
 from neutron_lib import constants
 from neutron_lib import context
@@ -48,17 +49,22 @@ class OvnDbSynchronizer(metaclass=abc.ABCMeta):
         self.ovn_driver = ovn_driver
         self.ovn_api = ovn_api
         self.core_plugin = core_plugin
+        self._thread = None
 
     def sync(self, delay_seconds=10):
-        self._gt = greenthread.spawn_after_local(delay_seconds, self.do_sync)
+        time.sleep(delay_seconds)
+        self._thread = threading.Thread(target=self.do_sync)
+        self._thread.start()
 
     @abc.abstractmethod
     def do_sync(self):
         """Method to sync the OVN DB."""
 
     def stop(self):
+        # TODO(ralonsoh): this method a the function executed in the thread
+        # should support a fast exit way.
         try:
-            self._gt.kill()
+            self._thread.join()
         except AttributeError:
             # Haven't started syncing
             pass
@@ -67,10 +73,12 @@ class OvnDbSynchronizer(metaclass=abc.ABCMeta):
 class OvnNbSynchronizer(OvnDbSynchronizer):
     """Synchronizer class for NB."""
 
-    def __init__(self, core_plugin, ovn_api, sb_ovn, mode, ovn_driver):
+    def __init__(self, core_plugin, ovn_api, sb_ovn, mode, ovn_driver,
+                 is_maintenance=False):
         super().__init__(
             core_plugin, ovn_api, ovn_driver)
         self.mode = mode
+        self.is_maintenance = is_maintenance
         self.l3_plugin = directory.get_plugin(plugin_constants.L3)
         self.pf_plugin = directory.get_plugin(plugin_constants.PORTFORWARDING)
         if not self.pf_plugin:
@@ -119,6 +127,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         self.sync_routers_and_rports(ctx)
         self.sync_port_qos_policies(ctx)
         self.sync_fip_qos_policies(ctx)
+        self.sync_fip_dnat_rules()
 
         LOG.debug("OVN-Northbound DB sync process completed @ %s",
                   str(datetime.now()))
@@ -1380,12 +1389,25 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         LOG.debug('OVN-NB Sync port QoS policies started @ %s',
                   str(datetime.now()))
         ovn_qos_ext = ovn_qos.OVNClientQosExtension(nb_idl=self.ovn_api)
-        with db_api.CONTEXT_READER.using(ctx), \
-                self.ovn_api.transaction(check_error=True) as txn:
+        _ports = []
+        with db_api.CONTEXT_READER.using(ctx):
             for port in self.core_plugin.get_ports(ctx):
                 if not ovn_qos_ext.port_effective_qos_policy_id(port)[0]:
                     continue
-                ovn_qos_ext.create_port(txn, port, None)
+                _ports.append(port)
+
+        if not _ports:
+            # Nothing to do.
+            pass
+        elif not (self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR or
+                  self.is_maintenance):
+            for port in _ports:
+                LOG.warning('Port QoS policy missing in OVN NB DB, port_id=%s',
+                            port['id'])
+        else:
+            with self.ovn_api.transaction(check_error=True) as txn:
+                for port in _ports:
+                    ovn_qos_ext.create_port(txn, port, None)
 
         LOG.debug('OVN-NB Sync port QoS policies completed @ %s',
                   str(datetime.now()))
@@ -1395,16 +1417,57 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         LOG.debug('OVN-NB Sync Floating IP QoS policies started @ %s',
                   str(datetime.now()))
         ovn_qos_ext = ovn_qos.OVNClientQosExtension(nb_idl=self.ovn_api)
-        with db_api.CONTEXT_READER.using(ctx), \
-                self.ovn_api.transaction(check_error=True) as txn:
+        _fips = []
+        with db_api.CONTEXT_READER.using(ctx):
             for fip in self.l3_plugin.get_floatingips(ctx):
                 if not fip.get('qos_policy_id'):
                     continue
-                ovn_qos_ext.create_floatingip(txn, fip)
+                _fips.append(fip)
+
+        if not _fips:
+            # Nothing to do.
+            pass
+        elif not (self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR or
+                  self.is_maintenance):
+            for fip in _fips:
+                LOG.warning('Floating IP QoS policy missing in OVN NB DB, '
+                            'fip_id=%s', fip['id'])
+        else:
+            with self.ovn_api.transaction(check_error=True) as txn:
+                for fip in _fips:
+                    ovn_qos_ext.create_floatingip(txn, fip)
 
         LOG.debug('OVN-NB Sync Floating IP QoS policies completed @ %s',
                   str(datetime.now()))
 
+    def sync_fip_dnat_rules(self):
+        """Sync all FIPs NAT rules, setting the configured stateless option"""
+        LOG.debug('OVN-NB Sync Floating IP NAT rules started @ %s',
+                  str(datetime.now()))
+        stateless_nat = ('true' if ovn_conf.is_stateless_nat_enabled() else
+                         'false')
+        nat_rules = []
+        for nat_rule in self.ovn_api.get_floatingips():
+            if nat_rule.get('options', {}).get('stateless') != stateless_nat:
+                nat_rules.append(nat_rule)
+
+        if not nat_rules:
+            # Nothing to do.
+            pass
+        elif not (self.mode == ovn_const.OVN_DB_SYNC_MODE_REPAIR or
+                  self.is_maintenance):
+            LOG.warning('The floating IP NAT rules must be updated to match '
+                        'the ``stateless_nat_enabled`` configuration flag.')
+        else:
+            with self.ovn_api.transaction(check_error=True) as txn:
+                for nat_rule in nat_rules:
+                    txn.add(self.ovn_api.db_set(
+                        'NAT', nat_rule['_uuid'],
+                        ('options', {'stateless': stateless_nat})))
+
+        LOG.debug('OVN-NB Sync Floating IP NAT rules completed @ %s',
+                  str(datetime.now()))
+
 
 class OvnSbSynchronizer(OvnDbSynchronizer):
     """Synchronizer class for SB."""
